void addRange(String s, float x, float y, float min, float max){
  Range obj = cp5.addRange(s)
     // disable broadcasting since setRange and setRangeValues will trigger an event
     .setBroadcast(false) 
     .setPosition(x,y)
     .setSize(400,20)
     .setHandleSize(5)
     .setRange(min,max)
     // after the initialization we turn broadcast back on again
     .setBroadcast(true)
     .setColorForeground(color(foreground, 200))
     .setColorBackground(background)
     .setColorValue(lerpColor(background, foreground, 0.3))
     .setColorValueLabel(lerpColor(background, foreground, 0.3))
     .setColorCaptionLabel(foreground)
     .setColorActive(color(255, 0, 0));
     
  if(s == "range_temp") obj.setCaptionLabel("Temperature (F)");
  if(s == "range_wind") obj.setCaptionLabel("Wind Strength (mph)");   
  if(s == "range_freq") obj.setCaptionLabel("Frequency");   
}

void addBang(String s, float x, float y){
  cp5.addBang(s)
     .setPosition(x, y)
     .setSize(15, 15)
     .setLabel(s)
     .setColorForeground(color(foreground, 200))
     .setColorBackground(background)
     .setColorCaptionLabel(foreground)
     .setColorActive(color(255, 0, 0));
}

void constructGUI() {
  cp5.addTextfield("input")
    .setPosition(width-15-150, 15)
    .setSize(150, 30)
    .setFont(createFont("SansSerif", 15))
    .setColor(background)
    .setColorActive(color(foreground))
    .setColorBackground(color(foreground, 200))
    .setColorForeground(background)
    .setColorCaptionLabel(foreground)
    .setColorCursor(background)
    .getCaptionLabel().setVisible(false);
    
  cp5.addScrollableList("dropdown")
     .setPosition(width-15-150, 50)
     .setSize(150, height-100)
     .setBarHeight(20)
     .setItemHeight(20)
     .addItems(selectedKeywords)
     .setFont(createFont("SansSerif", 10))
     .setType(ControlP5.LIST)
     .setColorActive(color(foreground))
     .setColorBackground(color(foreground, 200))
     .setColorForeground(background)
     .setColorCaptionLabel(foreground)
     .setColorValue(background)
     .getCaptionLabel().setVisible(false);
  
  addRange("range_freq", 15, height-65 , 0,globalMaxMeanFrequency);
  addRange("range_wind", 15, height-30, globalMinMeanWindStr, globalMaxMeanWindStr); 
  addRange("range_temp", 15, height-100, globalMinMeanTemp, globalMaxMeanTemp);

  addBang("Temperature View", 15, height-213);
  addBang("Frequency View", 15, height-177);
  addBang("Date View", 15, height-142);

  addBang("Toggle Color Search", 105, height-142);
  addBang("Toggle Auto Rotation", 105, height-177);
  addBang("Invert Colors", 105, height-213);

  //make control panel stay
  cp5.setAutoDraw(false);
}

void drawGUI() {
  hint(DISABLE_DEPTH_TEST);
  cam.beginHUD();
  
  stats();
  legend();

  textSize(10);
  textAlign(RIGHT, BOTTOM);
  text("MERT TOKA || MAT259 - DATA VISUALIZATION (WINTER 2017)", width-15, height-15);
  textValue = cp5.get(Textfield.class, "input").getText();
  if(textValue.equals("")){
    textAlign(LEFT, TOP);  
    text("SEATTLE PUBLIC LIBRARY CHECKOUTS\nAND HISTORICAL WEATHER DATA FUSION", 15, 15);
  }
  cp5.draw();

  cam.endHUD();
  hint(ENABLE_DEPTH_TEST);
}

void stats(){
  if(selectedKeywords.size() == 0)
    return;
  
  Keyword w = keywordMap.get(selectedKeywords.get(0));
  Date d = new java.util.Date((long)w.mean_time);
  DateFormat dateFormat = new SimpleDateFormat("MM/dd/yyyy");
  
  noStroke();
  fill(foreground, 230);
  rect(0, 15, (textWidth(w.word)+15)*3, 80);
  textSize(45);
  fill(background);
  textAlign(LEFT, CENTER);
  text(w.word, 20, 48);
  
  fill(foreground);
  textSize(20);
  textLeading(21);
  text("Average Temperature:\n  "+nfc(w.mean_temp,2)+" F,\n  "+nfc((w.mean_temp - 32) * 5 / 9, 2)+" C", 15, 150);
  text("Average Frequency:\n  "+nfc(w.mean_freq, 2), 15, 210);
  text("Average Wind:\n  "+nfc(w.mean_wind_str,2)+ " mph ["+pickOrientation(w.mean_wind_dir) +"]\n  " +
                           nfc(w.mean_wind_str * 1.609344,2) + " kph ["+pickOrientation(w.mean_wind_dir) + "]", 15, 270);
  text("Total Usage:\n  "+ int(w.count*w.mean_freq), 15, 332);
  text("Average Date:\n  "+dateFormat.format(d), 15, 378);
  text("Time Barcode:", 15, 421);

  for(int i = 0; i < w.references.size(); i++){
    stroke(foreground, 200);
    strokeWeight(2);
    pushMatrix();
    translate(15, 440);
    line(map(w.references.get(i).date.getTime(), 
              beginDate.getTime(),
              endDate.getTime(),
              0, 300), 0, 
         map(w.references.get(i).date.getTime(), 
              beginDate.getTime(),
              endDate.getTime(),
              0, 300), 50);
    popMatrix();
  }
  pushMatrix();
  translate(15, 440);
  stroke(foreground, 30);
  line(0, 0, 0, 50);
  line(300, 0, 300, 50);
  
  stroke(255,0,0, 150);
  line(map(d.getTime(), beginDate.getTime(),
              endDate.getTime(),
              0, 300), 0, 
       map(d.getTime(), beginDate.getTime(),
              endDate.getTime(),
              0, 300), 50);
  popMatrix();
  strokeWeight(1);
}

void legend() {
  if(!useColors)
    return;  
    
  float margin = 15;
  float cellWidth = 15;
  pushMatrix();
  translate(0, -218);
  for (int i = -5; i < 95; i+=10) {
    color c = pickColor(i);
    stroke(0);
    fill(c);
    rect(margin, height-margin-cellWidth-((i+5)/10.)*cellWidth, cellWidth, cellWidth);
    fill(foreground);
    textAlign(LEFT, TOP);
    textSize(12);
    text((i-5) + "-" + (i+5), margin+cellWidth+5, height-margin-cellWidth-((i+5)/10.)*cellWidth);
  }
  popMatrix();
}