class Keyword{
  String word;
  float mean_temp;
  float mean_freq;
  float mean_wind_dir;
  float mean_wind_str;
  long mean_time;
  int count; // same as references.size()
  
  ArrayList<Hour> references;
  
  Keyword(){
    references = new ArrayList<Hour>();
    count = 0;
    word = "";
    
    mean_temp = 0;
    mean_freq = 0;
    mean_wind_dir = 0;
    mean_wind_str = 0;
    mean_time = 0;
  }
  
  void addInstance(String w, float t, float f, float wD, float wS, Hour ref){
    word = w;
    references.add(ref);
    mean_time += ref.date.getTime();
    count++;
    mean_freq += f;
    mean_temp += t;
    mean_wind_dir += wD;
    mean_wind_str += wS;
  }
  
  void calculateAverage(){
    if(count != 0){
      mean_freq /= count;
      mean_temp /= count;
      mean_wind_dir /= count;
      mean_wind_str /= count;
      mean_time /= count;
    }
  }
};

// Load Library Table
void readLibraryTable(){
  DateFormat df = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");  
  for(int i = 0; i < keywordCSV.getRowCount()-1; i++){
    Date date = null,
         dateN= null;
    try {
      // Parse current and next date
      // 
      date = df.parse(keywordCSV.getString(i, 0));
      dateN = df.parse(keywordCSV.getString(i+1, 0));
      
      // Determine max and min total checkouts
      globalMaxAmount = max(keywordCSV.getFloat(i, 1), globalMaxAmount);
      globalMinAmount = min(keywordCSV.getFloat(i, 1), globalMinAmount);
      
      // Get weather information for current date for next block of time (6 hours)
      float sum_temp = 0,
            sum_direction = 0,
            sum_strength = 0;
      int count = 0;
      // for each time between date and dateN ...
      for(int j = 0; j < (dateN.getTime()-date.getTime()) / (60 * 60 * 1000); j++){
        // get weather information
        Weather val = weatherMap.get(new Date(date.getTime()+j*60*60*1000));
        // if there is an entry for spesific date in weatherMap sum them up
        if(val != null){
          count++;
          sum_temp += val.temp;
          sum_direction += val.windDir;
          sum_strength += val.windStr;
        }  
      }
      // if we have enough information ...
      if(count != 0){
        // Create hour object with average weather information
        Hour h = new Hour(date, 
                          keywordCSV.getFloat(i, 1), 
                          sum_temp/count, 
                          sum_direction/count, 
                          sum_strength/count);
                          
        // Process its keywords
        String[] key_pairs = split(keywordCSV.getString(i, 2), '#');
        for (int j = 0; j < key_pairs.length; j++){
          if(split(key_pairs[j], ' ').length >= 2){
            String s = split(key_pairs[j], ' ')[0];
            float f = parseFloat(split(key_pairs[j], ' ')[1]);
            globalMaxFrequency = max(f, globalMaxFrequency);
            
            // add word and frequency to hour objects array
            h.addWord(s, f);
            
            if(s.length() > 2){
              // add Keyword obj to keywordMap if it doesnt exist in it
              if(!keywordMap.containsKey(s)){
                keywordMap.put(s, new Keyword());
              }
              // update its values if exist
              else{
                keywordMap.get(s).addInstance(s,
                                              sum_temp/count,
                                              f,
                                              sum_direction/count, 
                                              sum_strength/count,
                                              h);
              }
            }
          }
        }
        // add hour to the array
        hoursArray.add(h);
      }
    } catch (Exception e) { e.printStackTrace(); }
  }
  
  int count = 0;
  // Calculates average values for each keyword
  //
  for(Keyword k: keywordMap.values()){
    k.calculateAverage();
    
    Collections.sort(k.references);    
    
    count++;
    
    // calculates boundary cases
    globalMaxMeanFrequency = sliderMaxMeanFrequency = max(k.mean_freq, globalMaxMeanFrequency);
    sliderMinMeanFrequency = 1;
    globalMaxMeanTemp = sliderMaxMeanTemp = max(k.mean_temp, globalMaxMeanTemp);
    globalMinMeanTemp = sliderMinMeanTemp = min(k.mean_temp, globalMinMeanTemp);
    globalMaxMeanWindStr = sliderMaxMeanWind = max(k.mean_wind_str, globalMaxMeanWindStr);
    globalMinMeanWindStr = sliderMinMeanWind = min(k.mean_wind_str, globalMinMeanWindStr);
    globalMaxCount = max(k.count, globalMaxCount);
  }
  println("Total number of keywords: "+ count + "\n  between " + df.format(beginDate) + " and \n          " + df.format(endDate));
}