void setup(){
  fullScreen(P3D, 1);
  //size(1600, 950, P3D);
 
  font = createFont("SansSerif", 60);
  textFont(font);
  
  cp5 = new ControlP5(this);
   
  cam = new PeasyCam(this, 1500);
  singleDragHandler = cam.getRotateDragHandler();
  
  cam.lookAt(500, 500, 500);
  cam.setRotations(0.68, 0.65, -0.52);
  cam.setMinimumDistance(0);
  cam.setMaximumDistance(2000);
 
  weatherCSV = loadTable("weather_05_17.csv", "header");
  keywordCSV = loadTable("nltk_05_17.csv");
  
  findTimeBounds();
  
  readWeatherTable(); // constructs Weather hashmap
  readLibraryTable(); // constructs Keyword hashmap and Hour array 
  
  constructGUI();
}

float prevTime = second();

void draw(){
  if(mouseY > height - 130 && mouseX < width/3)
    cam.setLeftDragHandler(null);
  else
    cam.setLeftDragHandler(singleDragHandler);
    
  background(background);
  if(autorotate)
    cam.rotateY(TWO_PI/1000);
  
  selectedKeywords.clear();
  for(Keyword w : keywordMap.values()){
    if(w.mean_freq < sliderMinMeanFrequency || w.mean_freq > sliderMaxMeanFrequency)
      continue;
    if(w.mean_temp < sliderMinMeanTemp || w.mean_temp > sliderMaxMeanTemp)
      continue;
    if(w.mean_wind_str < sliderMinMeanWind || w.mean_wind_str > sliderMaxMeanWind)
      continue;
      
    float[] cen = getXYZ(w);
    //float[] pos = cam.getPosition();
    float[] rot = cam.getRotations();
    
    // rotation and basis
    PVector p = new PVector(rot[0], rot[1], rot[2]); 
    float[] n = p.normalize().array();
    
    //float dist = sqrt(pow(cen[0]-pos[0],2) + 
    //                  pow(cen[1]-pos[1],2) +
    //                  pow(cen[2]-pos[2],2));
    
    pushMatrix();
    translate(cen[0], cen[1], cen[2]);
    rotateX(rot[0]);
    rotateY(rot[1]);
    rotateZ(rot[2]);
    try{
      if(textValue.equals("")){
        cp5.get(ScrollableList.class, "dropdown").hide();
        fill(foreground, 142);
        textSize(8);
      }
      else if(match(w.word, textValue) != null){
        cp5.get(ScrollableList.class, "dropdown").show();
        if(useColors) fill(pickColor(w.mean_temp));
        else          fill(foreground);
        textSize(20);
        selectedKeywords.add(w.word);
        cp5.get(ScrollableList.class, "dropdown").setItems(selectedKeywords);
      }
      else{
        fill(foreground, 100);
        textSize(3);
      }
    }
    catch(Exception e){
      fill(pickColor(w.mean_temp));
      textSize(8);
    }
    if(selectedKeywords.size() == 0) 
        cp5.get(ScrollableList.class, "dropdown").hide();
    textAlign(CENTER, CENTER);
    text(w.word, 0, 0);
    popMatrix();
  }
  
  drawPeripharils(true, true);
  drawGUI();
}

float[] getXYZ(Keyword w){
  float[] arr = new float[3];
  arr[0] = map(w.mean_temp, globalMinMeanTemp, globalMaxMeanTemp, 0, 1000);
  arr[1] = map(w.mean_time, beginDate.getTime(), endDate.getTime(), 10, 1000);
  arr[2] = logmap(w.mean_freq, 1, globalMaxMeanFrequency, 0, 1000);
  return arr;
}