////////// MATH
float logmap(float v, float istart, float istop, float ostart, float ostop){
  float result = 0;
  try{
    result = map(log(v), log(max(1,istart)), log(istop), ostart, ostop);
  }catch(Exception e) {/*e.printStackTrace();*/ }
  return result;
}

@Deprecated
float convertDateToFloat(Date d, float offset){
  float diff = (endDate.getTime() - beginDate.getTime()) / (60*60*1000);
  return map((d.getTime()-beginDate.getTime()) / (60*60*1000), 0, diff, 0, diff*offset);
}

////////// DRAWING
// Draws the axis on origin
void drawAxis(float scaleX, float scaleY, float scaleZ) {
  stroke(255, 0, 0);
  line(0, 0, 0, scaleX, 0, 0);
  stroke(0, 255, 0);
  line(0, 0, 0, 0, scaleY, 0);
  stroke(0, 0, 255);
  line(0, 0, 0, 0, 0, scaleZ);
}
void drawPeripharils(boolean isBox, boolean isLabels){
  if(isBox){
    pushMatrix();
    noFill();
    stroke(foreground, 10);
    translate(500,500,500);
    box(1000);
    popMatrix();
  }
  
  if(isLabels){
    labelTicking(10, false, globalMinMeanTemp, globalMaxMeanTemp, 
                 new PVector(1,0,0), new PVector(0,1,0), new PVector(0, 0, 0));
    labelTicking(10, true, 1, globalMaxMeanFrequency, 
                 new PVector(0,0,1), new PVector(0,1,0), new PVector(0, 1, 0));
    labelTicking(10, false, beginDate.getTime(), endDate.getTime(), 
                 new PVector(0,1,0), new PVector(1,0,0), new PVector(0, 0, 1));
    
    float size = 25;
    fill(foreground);
    textSize(25);
    textAlign(RIGHT, BOTTOM);
    pushMatrix();
    translate(990, -size, 0);
    text("Temperature (F)", 0, 0);
    popMatrix();
    
    textAlign(LEFT, BOTTOM);
    pushMatrix();
    translate(0, -size, 990);
    rotate(HALF_PI, 0, 1, 0);
    text("Frequency", 0, 0);
    popMatrix();
    
    pushMatrix();
    translate(-size, 990, 0);
    rotate(3*HALF_PI, 0, 0, 1);
    text("Average Date", 0, 0);
    popMatrix();
  }
}
// Draws ticks and their labels
// resolution = number of ticks
// isLog      = whether axis is in log scale
// min-max    = min and max values of axis
// p          = which axis
// p2         = labeling offset
// p3         = rotation
void labelTicking(int resolution, boolean isLog, float min, float max, PVector p, PVector p2, PVector p3){  
  int newres = resolution*5;
  for(int i = 0; i < newres; i++){
    float range = (max-min)/newres;
    float c = isLog ? logmap(min+i*range, min, max, 0, 1000) : map(min+i*range, min, max, 0, 1000);
    
    float size = 8;
    if(i%resolution==resolution-1) {
      stroke(foreground); 
      size = 5;
    }
    else {
      stroke(foreground, 100);
      size = 3.5;
    }
    line(p.x*c-size, p.y*c, p.z*c,  p.x*c+size, p.y*c, p.z*c);
    line(p.x*c, p.y*c-size, p.z*c,  p.x*c, p.y*c+size, p.z*c);
    line(p.x*c, p.y*c, p.z*c-size,  p.x*c, p.y*c, p.z*c+size);
    
    if(i%resolution==resolution-1) {
      size = 10;
      fill(foreground);
      textSize(12);
      textAlign(CENTER, CENTER);
      pushMatrix();
      translate(p.x*c-size*p2.x, p.y*c-size*p2.y, p.z*c-size*p2.z);
      rotate(p3.z == 1 ? 3*HALF_PI : HALF_PI, p3.x, p3.y, p3.z);
      if(p3.z == 1)
      {
        Date d = new java.util.Date((long)(min+i*range));
        DateFormat dateFormat = new SimpleDateFormat("MM/dd/yyyy");
        text(dateFormat.format(d), 0, 0);
        
      }
      else
        text(nfc(min+i*range, 1), 0, 0);
      popMatrix();
    }
  }
}
color pickColor(float temp){
  if(temp > 90)
    return color(#B10909);
  else if(temp > 80)
    return color(#C53600);
  else if(temp > 70)
    return color(#DD531E);
  else if(temp > 60)
    return color(#EC9800);
  else if(temp > 50)
    return color(#FECF3B);
  else if(temp > 40)
    return color(#2DC558);
  else if(temp > 30)
    return color(#03902B);
  else if(temp > 20)
    return color(#0772B8);
  else if(temp > 10)
    return color(#3993CE);
  else if(temp >= 0)
    return color(#A496C0);
  else
    return color(#FFFFFF);
}
String pickOrientation(float degree){
  if(degree <= 22.5)         return "N";
  else if(degree <= 22.5*3)  return "NE";
  else if(degree <= 22.5*5)  return "E";
  else if(degree <= 22.5*7)  return "SE";
  else if(degree <= 22.5*9)  return "S";
  else if(degree <= 22.5*11) return "SW";
  else if(degree <= 22.5*13) return "W";
  else if(degree <= 22.5*15) return "NW";
  else                       return "N";
}

////////// CSV HELPER FUNCTION
float safeGetFloat(Table t, int i, String c){
  float num = t.getFloat(i, c);
  if(Float.isNaN(num))
    return 0;
  return num;
}


void findTimeBounds(){
  DateFormat df = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");  
  try {
    beginDate = df.parse(keywordCSV.getString(0, 0));
    endDate = df.parse(keywordCSV.getString(keywordCSV.getRowCount()-1, 0));
  }catch(Exception e){}
}